@@ -238,6 +238,27 @@ proc_spawn_process(char *lpszwapper, fcgid_proc_info * procinfo,
 		}
 	}
 
+	{
+		int oldflags = fcntl(unix_socket, F_GETFD, 0);
+
+		if (oldflags < 0) {
+			ap_log_error(APLOG_MARK, APLOG_WARNING, apr_get_os_error(),
+						 procinfo->main_server,
+						 "mod_fcgid: fcntl F_GETFD failed");
+			close(unix_socket);
+			return errno;
+		}
+
+		oldflags |= FD_CLOEXEC;
+		if (fcntl(unix_socket, F_SETFD, oldflags) < 0) {
+			ap_log_error(APLOG_MARK, APLOG_WARNING, apr_get_os_error(),
+						 procinfo->main_server,
+						 "mod_fcgid: fcntl F_SETFD failed");
+			close(unix_socket);
+			return errno;
+		}
+	}
+
 	/* Build environment variables */
 	proc_environ = ap_create_environment(procnode->proc_pool,
 										 procinfo->proc_environ);
@@ -540,7 +561,7 @@ apr_status_t proc_read_ipc(server_rec * main_server,
 		}
 	} while (retcode == -1 && APR_STATUS_IS_EINTR(errno));
 	if (retcode == -1 && !APR_STATUS_IS_EAGAIN(errno)) {
-		ap_log_error(APLOG_MARK, APLOG_INFO, errno,
+		ap_log_error(APLOG_MARK, APLOG_WARNING, errno,
 					 main_server,
 					 "mod_fcgid: read data from fastcgi server error");
 		return errno;
@@ -554,12 +575,12 @@ apr_status_t proc_read_ipc(server_rec * main_server,
 		retcode = poll(pollfds, 1, ipc_handle->communation_timeout * 1000);
 	} while (retcode <= 0 && APR_STATUS_IS_EINTR(errno));
 	if (retcode == -1) {
-		ap_log_error(APLOG_MARK, APLOG_ERR, errno,
+		ap_log_error(APLOG_MARK, APLOG_WARNING, errno,
 					 main_server,
 					 "mod_fcgid: poll unix domain socket error");
 		return errno;
 	} else if (retcode == 0) {
-		ap_log_error(APLOG_MARK, APLOG_INFO, 0,
+		ap_log_error(APLOG_MARK, APLOG_WARNING, 0,
 					 main_server,
 					 "mod_fcgid: read data timeout in %d seconds",
 					 ipc_handle->communation_timeout);
@@ -574,13 +595,13 @@ apr_status_t proc_read_ipc(server_rec * main_server,
 	} while (retcode == -1 && APR_STATUS_IS_EINTR(errno));
 
 	if (retcode == 0) {
-		ap_log_error(APLOG_MARK, APLOG_INFO, 0,
+		ap_log_error(APLOG_MARK, APLOG_WARNING, 0,
 					 main_server,
 					 "mod_fcgid: Read data error, fastcgi server has close connection");
 		return APR_EPIPE;
 	}
 
-	ap_log_error(APLOG_MARK, APLOG_INFO, errno,
+	ap_log_error(APLOG_MARK, APLOG_WARNING, errno,
 				 main_server,
 				 "mod_fcgid: read data from fastcgi server error.");
 	return errno;
