@@ -148,8 +148,7 @@ void procmgr_init_spawn_cmd(fcgid_command * command, request_rec * r,
 	/* Update fcgid_command with wrapper info */
 	command->wrapperpath[0] = '\0';
 	if ((wrapperconf = get_wrapper_info(argv0, r))) {
-		strncpy(command->wrapperpath, wrapperconf->wrapper_path,
-				_POSIX_PATH_MAX);
+		strncpy(command->wrapperpath, wrapperconf->path, _POSIX_PATH_MAX);
 		command->wrapperpath[_POSIX_PATH_MAX - 1] = '\0';
 		command->deviceid = wrapperconf->deviceid;
 		command->inode = wrapperconf->inode;
