@@ -19,6 +19,7 @@ static int g_variables_inited = 0;
 static int g_busy_timeout;
 static int g_connect_timeout;
 static int g_comm_timeout;
+static int g_max_requests_per_process;
 
 static fcgid_procnode *apply_free_procnode(server_rec * main_server,
 										   fcgid_command * command)
@@ -106,6 +107,35 @@ return_procnode(server_rec * main_server,
 	safe_unlock(main_server);
 }
 
+static int
+count_busy_processes(server_rec * main_server, fcgid_command * command)
+{
+	int result = 0;
+	fcgid_procnode *previous_node, *current_node, *next_node;
+	fcgid_procnode *proc_table = proctable_get_table_array();
+	fcgid_procnode *busy_list_header = proctable_get_busy_list();
+
+	safe_lock(main_server);
+
+	previous_node = busy_list_header;
+	current_node = &proc_table[previous_node->next_index];
+	while (current_node != proc_table) {
+		if (current_node->inode == command->inode
+			&& current_node->deviceid == command->deviceid
+			&& current_node->share_grp_id == command->share_grp_id
+			&& current_node->uid == command->uid
+			&& current_node->gid == command->gid) {
+			result++;
+		}
+		next_node = &proc_table[current_node->next_index];
+		current_node = next_node;
+	}
+
+	safe_unlock(main_server);
+
+	return result;
+}
+
 apr_status_t bucket_ctx_cleanup(void *thectx)
 {
 	/* Cleanup jobs:
@@ -145,6 +175,12 @@ apr_status_t bucket_ctx_cleanup(void *thectx)
 			ctx->procnode->diewhy = FCGID_DIE_COMM_ERROR;
 			return_procnode(main_server, ctx->procnode,
 							1 /* communication error */ );
+		} else if (g_max_requests_per_process != -1
+				   && ++ctx->procnode->requests_handled >=
+				   g_max_requests_per_process) {
+			ctx->procnode->diewhy = FCGID_DIE_LIFETIME_EXPIRED;
+			return_procnode(main_server, ctx->procnode,
+							1 /* handled all requests */ );
 		} else
 			return_procnode(main_server, ctx->procnode,
 							0 /* communication ok */ );
@@ -249,6 +285,8 @@ handle_request(request_rec * r, const char *argv0,
 		g_connect_timeout = get_ipc_connect_timeout(r->server);
 		g_comm_timeout = get_ipc_comm_timeout(r->server);
 		g_busy_timeout = get_busy_timeout(r->server);
+		g_max_requests_per_process =
+			get_max_requests_per_process(r->server);
 		if (g_comm_timeout == 0)
 			g_comm_timeout = 1;
 		g_variables_inited = 1;
@@ -256,9 +294,9 @@ handle_request(request_rec * r, const char *argv0,
 
 	bucket_ctx = apr_pcalloc(request_pool, sizeof(*bucket_ctx));
 	if (!bucket_ctx) {
-		ap_log_error(APLOG_MARK, APLOG_WARNING, rv, r->server,
-					 "mod_fcgid: apr_calloc of %d bytes failed in handle_request function",
-					 sizeof(*bucket_ctx));
+		ap_log_error(APLOG_MARK, APLOG_WARNING, apr_get_os_error(),
+					 r->server,
+					 "mod_fcgid: apr_calloc bucket_ctx failed in handle_request function");
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
 	bucket_ctx->ipc.connect_timeout = g_connect_timeout;
@@ -271,7 +309,6 @@ handle_request(request_rec * r, const char *argv0,
 	for (i = 0; i < FCGID_REQUEST_COUNT; i++) {
 		/* Apply a free process slot, send a spawn request if I can't get one */
 		for (j = 0; j < FCGID_APPLY_TRY_COUNT; j++) {
-			int mpm_state = 0;
 			apr_ino_t inode =
 				wrapper_conf ? wrapper_conf->inode : r->finfo.inode;
 			apr_dev_t deviceid =
@@ -289,12 +326,18 @@ handle_request(request_rec * r, const char *argv0,
 			if (bucket_ctx->procnode)
 				break;
 
-			apr_sleep(apr_time_from_sec(1));
+			/* Avoid sleeping the very first time through if there are no
+			   busy processes; the problem is just that we haven't spawned
+			   anything yet, so waiting is pointless */
+			if (i > 0 || j > 0
+				|| count_busy_processes(r->server, &fcgi_request)) {
+				apr_sleep(apr_time_from_sec(1));
 
-			bucket_ctx->procnode =
-				apply_free_procnode(r->server, &fcgi_request);
-			if (bucket_ctx->procnode)
-				break;
+				bucket_ctx->procnode =
+					apply_free_procnode(r->server, &fcgi_request);
+				if (bucket_ctx->procnode)
+					break;
+			}
 
 			/* Send a spawn request if I can't get a process slot */
 			procmgr_post_spawn_cmd(&fcgi_request, r);
@@ -339,7 +382,7 @@ handle_request(request_rec * r, const char *argv0,
 		apr_brigade_create(request_pool, r->connection->bucket_alloc);
 	if (!brigade_stdout) {
 		ap_log_error(APLOG_MARK, APLOG_WARNING, rv, r->server,
-			"mod_fcgid: apr_brigade_create failed in handle_request function");
+					 "mod_fcgid: apr_brigade_create failed in handle_request function");
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
 	APR_BRIGADE_INSERT_TAIL(brigade_stdout,
@@ -391,7 +434,7 @@ handle_request(request_rec * r, const char *argv0,
 	return cond_status;
 }
 
-int bridge_request(request_rec * r, const char *argv0,
+int bridge_request(request_rec * r, int role, const char *argv0,
 				   fcgid_wrapper_conf * wrapper_conf)
 {
 	apr_pool_t *request_pool = r->main ? r->main->pool : r->pool;
@@ -416,7 +459,7 @@ int bridge_request(request_rec * r, const char *argv0,
 
 	/* Build the begin request and environ request, append them to output_brigade */
 	if (!build_begin_block
-		(r->server, r->connection->bucket_alloc, output_brigade)
+		(role, r->server, r->connection->bucket_alloc, output_brigade)
 		|| !build_env_block(r->server, envp, r->connection->bucket_alloc,
 							output_brigade)) {
 		ap_log_error(APLOG_MARK, APLOG_WARNING, 0,
